<?php

namespace Drupal\acquia_perz;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Theme\ActiveTheme;

/**
 * Perz helper class.
 */
class PerzHelper {

  /**
   * Returns a unique hash for the current site.
   *
   * @return string
   *   A unique site hash, containing only alphanumeric characters.
   */
  public static function getSiteHash() {
    if (!($hash = \Drupal::state()->get('acquia_perz.site_hash', FALSE))) {
      $base_url = \Drupal::requestStack()->getCurrentRequest()->getHost();
      $hash = substr(base_convert(hash('sha256', uniqid($base_url, TRUE)), 16, 36), 0, 6);
      \Drupal::state()->set('acquia_perz.site_hash', $hash);
    }
    return $hash;
  }

  /**
   * Returns a environment variable.
   *
   * @return string
   *   An acquia site environment.
   */
  public static function getSiteEnvironment() {
    // During Beta, only production environment is supported.
    return 'prod';
  }

  /**
   * Get a site domain.
   *
   * @return string
   *   Returns a site domain.
   */
  public static function getSiteDomain() {
    return \Drupal::requestStack()->getCurrentRequest()->getHost();
  }

  /**
   * Get a site domain with host.
   *
   * @return string
   *   Returns a site domain with host.
   */
  public static function getSiteDomainWithHost() {
    return \Drupal::requestStack()->getCurrentRequest()->getSchemeAndHttpHost();
  }

  /**
   * Get account id.
   *
   * @return string|null
   *   Returns account id.
   */
  public static function getAccountId() {
    $subscription = \Drupal::service('acquia_connector.subscription');
    $subscription_data = $subscription->getSubscription();
    if (isset($subscription_data['acquia_perz'])) {
      return $subscription_data['acquia_perz']['account_id'];
    }
    return NULL;
  }

  /**
   * Collect imagefields for a particular entity bundle.
   *
   * @param string $entity_type_id
   *   The entity type identifier.
   * @param string $bundle
   *   The bundle name.
   *
   * @return array
   *   The array of image fields for a given entity bundle.
   */
  public static function collectImageFields($entity_type_id, $bundle) {
    $image_fields = [];
    $field_definitions = \Drupal::service('entity_field.manager')->getFieldDefinitions($entity_type_id, $bundle);
    foreach ($field_definitions as $field_key => $field_definition) {
      if ($field_definition->getType() === 'image') {
        $image_fields[$field_key] = $field_definition->getLabel();
      }
    }
    return $image_fields;
  }

  /**
   * Remove/update data from settings configuration.
   *
   * @param string $entity_type_id
   *   The entity type id.
   * @param string $bundle
   *   The bundle of entity type.
   * @param string $view_mode
   *   The view_mode of bundle.
   * @param array $data
   *   User input data.
   * @param array $config_view_mode
   *   Config_view_mode of previous store data.
   */
  public static function removeViewModeFromConfig($entity_type_id, $bundle, $view_mode, array $data, array &$config_view_mode) {
    $entity_view_perz_config = array_filter($data['view_mode']);
    if ((!empty($config_view_mode) && isset($config_view_mode[$entity_type_id][$bundle])
        && array_key_exists($view_mode, $config_view_mode[$entity_type_id][$bundle]))
      && empty($entity_view_perz_config)) {
      if (count($config_view_mode[$entity_type_id][$bundle]) > 1) {
        unset($config_view_mode[$entity_type_id][$bundle][$view_mode]);
      }
      elseif (count($config_view_mode[$entity_type_id]) > 1) {
        unset($config_view_mode[$entity_type_id][$bundle]);
      }
      else {
        unset($config_view_mode[$entity_type_id]);
      }
    }
    else {
      if (!empty($entity_view_perz_config)) {
        $role['render_role'] = $data['render_role'];
        $preview_image = isset($data['acquia_perz_preview_image']) ? ['preview_image' => $data['acquia_perz_preview_image']] : [];
        $config_view_mode[$entity_type_id][$bundle][$view_mode] = array_merge($role, $preview_image);
      }
    }
  }

  /**
   * Run decision webhook.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public static function runDecisionWebhook(EntityInterface $entity = NULL) {
    $config = \Drupal::configFactory()->get('acquia_perz.settings');
    $decision_endpoint = $config->get('api.endpoint');
    if (empty($decision_endpoint)
      || \Drupal::moduleHandler()->moduleExists('acquia_perz_push')
      || !\Drupal::service('entity_helper')->isValidEntity($entity)) {
      return;
    }
    $data = [
      'account_id' => self::getAccountId(),
      'origin' => self::getSiteHash(),
      'environment' => self::getSiteEnvironment(),
      'domain' => self::getSiteDomain(),
      'entity_type' => $entity->getEntityTypeId(),
      'entity_uuid' => $entity->uuid(),
      'op' => 'create',
    ];
    \Drupal::service('client_factory')->pushEntity($data);
  }

  /**
   * Get Active Theme.
   *
   * @return \Drupal\Core\Theme\ActiveTheme
   *   The active theme object.
   */
  public static function getActiveTheme() {
    return \Drupal::service('theme.manager')->getActiveTheme();
  }

  /**
   * Get Default Theme.
   *
   * @return \Drupal\Core\Theme\ActiveTheme
   *   The active theme object.
   */
  public static function getActiveDefaultTheme() {
    $activeDefaultThemeName = \Drupal::configFactory()->get('system.theme')->get('default');
    return \Drupal::service('theme.initialization')->getActiveThemeByName($activeDefaultThemeName);
  }

  /**
   * Set Active Theme.
   *
   * @param \Drupal\Core\Theme\ActiveTheme $theme
   *   The active theme object.
   */
  public static function setActiveTheme(ActiveTheme $theme) {
    \Drupal::service('theme.initialization')->loadActiveTheme($theme);
    \Drupal::service('theme.manager')->setActiveTheme($theme);
  }

  /**
   * Safely switches to another account.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The account to switch to.
   */
  public static function switchAccountTo(AccountInterface $account) {
    \Drupal::service('account_switcher')->switchTo($account);
  }

  /**
   * Reverts to a previous account after switching.
   */
  public static function switchAccountBack() {
    \Drupal::service('account_switcher')->switchBack();
  }

}
