<?php

namespace Drupal\acquia_perz\Plugin\GraphQL\DataProducer\Entity;

use Drupal\acquia_perz\EntityHelper;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Cache\RefinableCacheableDependencyInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\graphql\Plugin\GraphQL\DataProducer\DataProducerPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Entity variations producer.
 *
 * @DataProducer(
 *   id = "query_entity_variations",
 *   name = @Translation("Load entity variations"),
 *   description = @Translation("Loads a list of entity variations."),
 *   produces = @ContextDefinition("any",
 *     label = @Translation("List of entity variations")
 *   ),
 *   consumes = {
 *     "entity_type_id" = @ContextDefinition("string",
 *       label = @Translation("Entity type Id")
 *     ),
 *     "entity_uuid" = @ContextDefinition("string",
 *       label = @Translation("Entity Uuid")
 *     )
 *   }
 * )
 */
class QueryEntityVariations extends DataProducerPluginBase implements ContainerFactoryPluginInterface {

  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * The entity type bundle info.
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected $entityTypeBundleInfo;

  /**
   * The entity repository.
   *
   * @var \Drupal\Core\Entity\EntityRepositoryInterface
   */
  protected $repository;

  /**
   * Database Service Object.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The acquia perz entity settings.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   * @see \Drupal\acquia_perz\Form\ContentPublishingForm
   */
  protected $entityConfig;

  /**
   * The CIS settings.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   * @see \Drupal\acquia_perz\Form\ContentPublishingForm
   */
  protected $cisSettings;

  /**
   * The Perz entity helper.
   *
   * @var \Drupal\acquia_perz\EntityHelper
   */
  protected $entityHelper;

  /**
   * {@inheritdoc}
   *
   * @codeCoverageIgnore
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('renderer'),
      $container->get('date.formatter'),
      $container->get('datetime.time'),
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('entity_field.manager'),
      $container->get('entity_type.bundle.info'),
      $container->get('entity.repository'),
      $container->get('database'),
      $container->get('acquia_perz.entity_helper')
    );
  }

  /**
   * Constructor.
   *
   * @param array $configuration
   *   The plugin configuration.
   * @param string $pluginId
   *   The plugin id.
   * @param mixed $pluginDefinition
   *   The plugin definition.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter service.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The entity field manager.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entity_type_bundle_info
   *   The entity type bundle info service.
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $repository
   *   The entity repository service.
   * @param \Drupal\Core\Database\Connection $database
   *   The database service.
   * @param \Drupal\acquia_perz\EntityHelper $entity_helper
   *   The entity helper service.
   */
  public function __construct(
    array $configuration,
    $pluginId,
    $pluginDefinition,
    RendererInterface $renderer,
    DateFormatterInterface $date_formatter,
    TimeInterface $time,
    ConfigFactoryInterface $config_factory,
    EntityTypeManagerInterface $entity_type_manager,
    EntityFieldManagerInterface $entity_field_manager,
    EntityTypeBundleInfoInterface $entity_type_bundle_info,
    EntityRepositoryInterface $repository,
    Connection $database,
    EntityHelper $entity_helper
  ) {
    parent::__construct($configuration, $pluginId, $pluginDefinition);
    $this->renderer = $renderer;
    $this->dateFormatter = $date_formatter;
    $this->time = $time;
    $this->entityConfig = $config_factory->get('acquia_perz.entity_config');
    $this->cisSettings = $config_factory->get('acquia_perz.settings');
    $this->entityTypeManager = $entity_type_manager;
    $this->entityFieldManager = $entity_field_manager;
    $this->entityTypeBundleInfo = $entity_type_bundle_info;
    $this->repository = $repository;
    $this->database = $database;
    $this->entityHelper = $entity_helper;
  }

  /**
   * Resolver.
   *
   * @param string $entity_type_id
   *   The entity type id.
   * @param string $entity_uuid
   *   The entity uuid.
   * @param \Drupal\Core\Cache\RefinableCacheableDependencyInterface $metadata
   *   The metadata.
   *
   * @return array
   *   List of entity variations.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function resolve(
    $entity_type_id,
    $entity_uuid,
    RefinableCacheableDependencyInterface $metadata) {
    $variations = [];
    $entity = $this->repository->loadEntityByUuid($entity_type_id, $entity_uuid);
    if (is_null($entity)) {
      throw new \Exception("Entity not found.");
    }
    $bundle = $entity->bundle();
    $available_view_modes = $this->entityConfig
      ->get("view_modes.{$entity_type_id}.{$bundle}")
      ?? [];

    if (empty($available_view_modes)) {
      throw new \Exception("{$entity_type_id}.{$bundle} is not available for the export.");
    }

    foreach (array_keys($available_view_modes) as $view_mode) {
      foreach ($entity->getTranslationLanguages() as $language) {
        $langcode = $language->getId();
        $translation = $entity->getTranslation($langcode);
        $variations[] = $this->entityHelper->getEntityVariation(
          $translation,
          $view_mode,
          $langcode
        );
      }
    }
    $metadata->addCacheTags($entity->getCacheTags());
    $metadata->addCacheTags($this->entityConfig->getCacheTags());
    $metadata->addCacheContexts($entity->getCacheContexts());
    $metadata->addCacheContexts($this->entityConfig->getCacheContexts());

    return $variations;
  }

}
