<?php

namespace Drupal\Tests\acquia_perz\Kernel;

use Drupal\Core\Datetime\Entity\DateFormat;
use Drupal\Core\Entity\Entity\EntityViewDisplay;
use Drupal\Core\Entity\Entity\EntityViewMode;
use Drupal\Core\Entity\EntityInterface;
use Drupal\language\Entity\ConfigurableLanguage;
use Drupal\Tests\acquia_perz\Traits\CustomParagraphsTestTrait;
use Drupal\Tests\acquia_perz\Traits\EntityHelperTestTrait;
use Drupal\Tests\acquia_perz\Traits\TaxonomyFieldCreationTrait;
use Drupal\Tests\acquia_perz\Traits\TextFieldCreationTrait;
use Drupal\Tests\EntityViewTrait;
use Drupal\Tests\paragraphs\FunctionalJavascript\ParagraphsTestBaseTrait;

/**
 * Tests for discovery endpoint.
 *
 * @group node
 */
class VariationsEndpointTest extends PerzGraphQLTestBase {

  use EntityViewTrait;
  use CustomParagraphsTestTrait;
  use ParagraphsTestBaseTrait;
  use EntityHelperTestTrait;
  use TextFieldCreationTrait;
  use TaxonomyFieldCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected $graphQLQuery = <<<GQL
  query getEntityVariations(\$entity_type_id: String!, \$entity_uuid: String!) {
    entity_variations(entity_type_id: \$entity_type_id, entity_uuid: \$entity_uuid) {
      content_uuid,
      label,
      content_type,
      view_mode,
      language,
      updated,
      rendered_data,
      relations {
        field,
        terms
      }
    }
  }
GQL;

  /**
   * {@inheritdoc}
   */
  protected $graphqlServer = NULL;

  /**
   * {@inheritdoc}
   */
  protected $nodeEntityTypeId = 'node';

  /**
   * {@inheritdoc}
   */
  protected $nodeBundle = 'news';

  /**
   * {@inheritdoc}
   */
  protected $viewMode = 'default';

  /**
   * {@inheritdoc}
   */
  protected $viewModeDefaultValue = [
    'render_role' => 'anonymous',
    'preview_image' => '',
  ];

  /**
   * {@inheritdoc}
   */
  protected $tagsFieldName = 'field_tags';

  /**
   * {@inheritdoc}
   */
  protected $materialsFieldName = 'field_materials';

  /**
   * {@inheritdoc}
   */
  protected $categoriesFieldName = 'field_categories';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    DateFormat::create([
      'id' => 'fallback',
      'label' => 'Fallback',
      'pattern' => 'Y-m-d',
    ])->save();
    $this->graphqlServer = $this->createGraphqlServer(
      'Perz',
      'perz',
      'export_entities',
      '/perz3'
    );
    // We need to see exception details in response.
    $this->graphqlServer->set('debug_flag', TRUE)->save();
    // Create current user and setup graphql permission.
    $this->setUpCurrentUser([], ["execute {$this->graphqlServer->id()} arbitrary graphql requests"]);
  }

  /**
   * Use case 0. Check use case when no entities are selected on perz form.
   *
   * Exception message should be returned.
   *
   * @throws \Exception
   */
  public function testExceptions() {
    $this->drupalCreateContentType([
      'type' => $this->nodeBundle,
      'name' => $this->nodeBundle,
    ]);

    // 1. Check exception when entity uuid doesn't exist.
    $result = $this->sendQuery(
      $this->graphQLQuery,
      $this->graphqlServer,
      [
        'entity_type_id' => $this->nodeEntityTypeId,
        'entity_uuid' => 'fake_uuid',
      ]
    );
    $this->assertEquals(200, $result->getStatusCode());
    $json_content = json_decode($result->getContent(), TRUE);

    if (isset($json_content['errors'])
      && isset($json_content['errors'][0])
      && isset($json_content['errors'][0]['debugMessage'])) {
      $this->assertEquals("Entity not found.",
        $json_content['errors'][0]['debugMessage']);
    }

    // 2. Check exception when entity exists but entity types is not
    // checked on Entity configuration form.
    $node = $this->drupalCreateNode(['type' => $this->nodeBundle]);
    $this->setUpPerzEntityTypes([]);
    $result = $this->sendQuery(
      $this->graphQLQuery,
      $this->graphqlServer,
      [
        'entity_type_id' => $node->getEntityTypeId(),
        'entity_uuid' => $node->uuid(),
      ]
    );
    $this->assertEquals(200, $result->getStatusCode());
    $json_content = json_decode($result->getContent(), TRUE);
    if (isset($json_content['errors'])
      && isset($json_content['errors'][0])
      && isset($json_content['errors'][0]['debugMessage'])) {
      $this->assertEquals("node.{$this->nodeBundle} is not available for the export.",
        $json_content['errors'][0]['debugMessage']);
    }
  }

  /**
   * Use case 1.
   *
   * Test simple node without custom view modes, translations.
   *
   * and taxonomy relations.
   */
  public function testSimpleNode() {
    $this->drupalCreateContentType([
      'type' => $this->nodeBundle,
      'name' => $this->nodeBundle,
    ]);
    $node = $this->drupalCreateNode(['type' => $this->nodeBundle]);
    $entity_types = [
      $this->nodeEntityTypeId => [
        $this->nodeBundle => [
          $this->viewMode => $this->viewModeDefaultValue,
        ],
      ],
    ];
    $this->setUpPerzEntityTypes($entity_types);

    $graphql_query_variables = [
      'entity_type_id' => $node->getEntityTypeId(),
      'entity_uuid' => $node->uuid(),
    ];

    $result = $this->sendQuery(
      $this->graphQLQuery,
      $this->graphqlServer,
      $graphql_query_variables
    );
    // var_dump(($result));.
    $this->assertEquals(200, $result->getStatusCode());
    $json_content = json_decode($result->getContent(), TRUE);

    $entity_variations = $json_content['data']['entity_variations'];

    $this->assertCount(1, $entity_variations);
    $this->assertSame($entity_variations[0]['label'], $node->label());
    $this->assertSame($entity_variations[0]['view_mode'], $this->viewMode);
    $this->assertSame($entity_variations[0]['content_uuid'], $node->uuid());
    $this->assertSame($entity_variations[0]['content_type'], $node->getEntityTypeId());
    $this->assertSame($entity_variations[0]['language'], $node->language()->getId());

  }

  /**
   * Use case 2.a.
   *
   * Check use case when paragraph is added to node but only node type.
   *
   * is activated to be returned in graphQL endpoint.
   */
  public function testNodeWithParagraph() {
    $this->drupalCreateContentType([
      'type' => $this->nodeBundle,
      'name' => $this->nodeBundle,
    ]);
    $node_title = 'News title1';
    $node_body = 'News body1';
    $paragraph_title = 'Paragraph title 1.';
    $paragraph_field_name = 'field_paragraph';
    $paragraph_type = 'header_test';

    $this->createParagraphType($paragraph_type);
    $this->addParagraphFieldInBundle($this->nodeEntityTypeId, $this->nodeBundle, $paragraph_field_name, $paragraph_type);

    $paragraph = $this->createParagraph(
      $paragraph_type,
      $paragraph_title,
      TRUE
    );
    $node = $this->createNodeWithParagraph(
      $this->nodeBundle,
      $node_title,
      $node_body,
      $paragraph_field_name,
      $paragraph
    );

    $entity_types = [
      $this->nodeEntityTypeId => [
        $this->nodeBundle => [
          $this->viewMode => $this->viewModeDefaultValue,
        ],
      ],
    ];
    $this->setUpPerzEntityTypes($entity_types);

    $graphql_query_variables = [
      'entity_type_id' => $node->getEntityTypeId(),
      'entity_uuid' => $node->uuid(),
    ];

    $result = $this->sendQuery(
      $this->graphQLQuery,
      $this->graphqlServer,
      $graphql_query_variables
    );
    $this->assertEquals(200, $result->getStatusCode());
    $json_content = json_decode($result->getContent(), TRUE);

    $entity_variations = $json_content['data']['entity_variations'];

    $this->assertCount(1, $entity_variations);
    $this->assertSame($entity_variations[0]['label'], $node->label());
    $this->assertSame($entity_variations[0]['view_mode'], $this->viewMode);
    $this->assertSame($entity_variations[0]['content_uuid'], $node->uuid());
    $this->assertSame($entity_variations[0]['content_type'], $node->getEntityTypeId());
    $this->assertSame($entity_variations[0]['language'], $node->language()->getId());

    $target_node = \Drupal::service('entity.repository')->loadEntityByUuid(
      $entity_variations[0]['content_type'],
      $entity_variations[0]['content_uuid']
    );

    // Because rendering node with paragraphs doesn't work in kernel tests -
    // we'll check that paragraph is available for node another way:
    // 1. Check entity reference chain.
    $this->assertSame(
      $paragraph_title,
      $target_node->get('field_paragraph')->entity->get('field_title')->value
    );
    // 2. Check that paragraph field is visible for node's view display.
    $view_display = \Drupal::service('entity_display.repository')->getViewDisplay($target_node->getEntityTypeId(), $target_node->bundle());
    $view_display_components = $view_display->getComponents();
    $this->assertNotEmpty($view_display_components[$paragraph_field_name]);
    // 3. Check that paragraph entity is rendered correctly.
    $target_paragraph = $target_node->get('field_paragraph')->entity;
    $elements = \Drupal::service('entity_type.manager')
      ->getViewBuilder('paragraph')
      ->view($target_paragraph, $this->viewMode);
    $paragraph_rendered_content = \Drupal::service('renderer')->renderPlain($elements);
    $this->assertStringContainsString($paragraph_title, $paragraph_rendered_content);
  }

  /**
   * Use case 2.b.
   *
   * Check use case when paragraph is added to node but only paragraph.
   *
   * type is activated to be returned in graphQL endpoint.
   */
  public function testParagraphOnly() {
    $this->drupalCreateContentType([
      'type' => $this->nodeBundle,
      'name' => $this->nodeBundle,
    ]);
    $node_title = 'News title1';
    $node_body = 'News body1';
    $paragraph_title = 'Paragraph title 1.';
    $paragraph_field_name = 'field_paragraph';
    $paragraph_type = 'header_test';
    $this->createParagraphType($paragraph_type);
    $this->addParagraphFieldInBundle($this->nodeEntityTypeId, $this->nodeBundle, $paragraph_field_name, $paragraph_type);

    $paragraph = $this->createParagraph(
      $paragraph_type,
      $paragraph_title,
      TRUE
    );
    $this->createNodeWithParagraph(
      $this->nodeBundle,
      $node_title,
      $node_body,
      $paragraph_field_name,
      $paragraph
    );

    $entity_types = [
      'paragraph' => [
        $paragraph_type => [
          $this->viewMode => $this->viewModeDefaultValue,
        ],
      ],
    ];
    $this->setUpPerzEntityTypes($entity_types);

    $graphql_query_variables = [
      'entity_type_id' => $paragraph->getEntityTypeId(),
      'entity_uuid' => $paragraph->uuid(),
    ];

    $result = $this->sendQuery(
      $this->graphQLQuery,
      $this->graphqlServer,
      $graphql_query_variables
    );
    $this->assertEquals(200, $result->getStatusCode());
    $json_content = json_decode($result->getContent(), TRUE);

    $entity_variations = $json_content['data']['entity_variations'];

    $this->assertCount(1, $entity_variations);
    $this->assertSame($entity_variations[0]['view_mode'], $this->viewMode);
    $this->assertSame($entity_variations[0]['content_uuid'], $paragraph->uuid());
    $this->assertSame($entity_variations[0]['content_type'], $paragraph->getEntityTypeId());
    $this->assertSame($entity_variations[0]['language'], $paragraph->language()->getId());
    $this->assertStringNotContainsString($paragraph_title, $entity_variations[0]['rendered_data']);
    $this->assertStringNotContainsString($node_title, $entity_variations[0]['rendered_data']);
    $this->assertStringNotContainsString($node_body, $entity_variations[0]['rendered_data']);
  }

  /**
   * Use case 3.
   *
   * Test node with 1 view mode & multiple translations (en, fr).
   */
  public function testTranslations() {
    $translation_langcode = 'fr';
    ConfigurableLanguage::createFromLangcode($translation_langcode)->save();
    $this->drupalCreateContentType([
      'type' => $this->nodeBundle,
      'name' => $this->nodeBundle,
    ]);
    \Drupal::service('content_translation.manager')->setEnabled(
      $this->nodeEntityTypeId,
      $this->nodeBundle,
      TRUE
    );
    $en_title = 'en news 1';
    $en_body = 'en article body';
    $fr_title = 'fr news 1';
    $fr_body = 'fr article body';
    $en_node = $this->drupalCreateNode([
      'type' => $this->nodeBundle,
      'title' => $en_title,
      'body' => ['value' => $en_body],
    ]);
    $en_node->addTranslation('fr', [
      'title' => $fr_title,
      'body' => ['value' => $fr_body],
    ]);
    $en_node->save();
    $fr_node = $en_node->getTranslation($translation_langcode);

    $entity_types = [
      $this->nodeEntityTypeId => [
        $this->nodeBundle => [
          $this->viewMode => $this->viewModeDefaultValue,
        ],
      ],
    ];
    $this->setUpPerzEntityTypes($entity_types);

    $graphql_query_variables = [
      'entity_type_id' => $en_node->getEntityTypeId(),
      'entity_uuid' => $en_node->uuid(),
    ];

    $result = $this->sendQuery(
      $this->graphQLQuery,
      $this->graphqlServer,
      $graphql_query_variables
    );
    $this->assertEquals(200, $result->getStatusCode());
    $json_content = json_decode($result->getContent(), TRUE);

    $entity_variations = $json_content['data']['entity_variations'];

    $this->assertCount(2, $entity_variations);
    $this->assertSame($entity_variations[0]['label'], $en_node->label());
    $this->assertSame($entity_variations[0]['view_mode'], $this->viewMode);
    $this->assertSame($entity_variations[0]['content_uuid'], $en_node->uuid());
    $this->assertSame($entity_variations[0]['content_type'], $en_node->getEntityTypeId());
    $this->assertSame($entity_variations[0]['language'], $en_node->language()->getId());
    $this->assertStringNotContainsString($en_node->get('body')->getString(), $entity_variations[0]['rendered_data']);

    $this->assertSame($entity_variations[1]['label'], $fr_node->label());
    $this->assertSame($entity_variations[1]['view_mode'], $this->viewMode);
    $this->assertSame($entity_variations[1]['content_uuid'], $fr_node->uuid());
    $this->assertSame($entity_variations[1]['content_type'], $fr_node->getEntityTypeId());
    $this->assertSame($entity_variations[1]['language'], $fr_node->language()->getId());
    $this->assertStringNotContainsString($fr_node->get('body')->getString(), $entity_variations[1]['rendered_data']);
  }

  /**
   * Use case 4.
   *
   * Test node with 1 view mode & multiple translations (en, fr).
   */
  public function testCustomViewModes() {
    $this->drupalCreateContentType([
      'type' => $this->nodeBundle,
      'name' => $this->nodeBundle,
    ]);

    // Textfield custom 1.
    $custom1_field_name = 'field_custom1';
    $field_storage = $this->createTextFieldStorage(
      $this->nodeEntityTypeId,
      $custom1_field_name
    );
    $this->createTextField(
      $field_storage,
      $this->nodeEntityTypeId,
      $this->nodeBundle
    );
    $custom1_view_mode_name = 'custom1';
    // Create custom1 view mode.
    EntityViewMode::create([
      'id' => "{$this->nodeEntityTypeId}.{$custom1_view_mode_name}",
      'label' => 'Custom 1',
      'targetEntityType' => $this->nodeEntityTypeId,
    ])->save();
    EntityViewDisplay::create([
      'targetEntityType' => $this->nodeEntityTypeId,
      'bundle' => $this->nodeBundle,
      'mode' => $custom1_view_mode_name,
    ])->setStatus(TRUE)
      ->setComponent($custom1_field_name, [
        'label' => 'inline',
        'type' => 'string',
      ])
      ->save();

    $en_title = 'en news 1';
    $en_body_value = 'en news 1 body';
    $en_text1_field_value = 'en news 1 text 1';
    $node = $this->drupalCreateNode([
      'type' => $this->nodeBundle,
      'title' => $en_title,
      'body' => ['value' => $en_body_value],
      $custom1_field_name => ['value' => $en_text1_field_value],
    ]);
    $node->save();

    $entity_types = [
      $this->nodeEntityTypeId => [
        $this->nodeBundle => [
          $this->viewMode => $this->viewModeDefaultValue,
          $custom1_view_mode_name => $this->viewModeDefaultValue,
        ],
      ],
    ];
    $this->setUpPerzEntityTypes($entity_types);

    $graphql_query_variables = [
      'entity_type_id' => $node->getEntityTypeId(),
      'entity_uuid' => $node->uuid(),
    ];

    $result = $this->sendQuery(
      $this->graphQLQuery,
      $this->graphqlServer,
      $graphql_query_variables
    );
    $this->assertEquals(200, $result->getStatusCode());
    $json_content = json_decode($result->getContent(), TRUE);

    $entity_variations = $json_content['data']['entity_variations'];

    // Default view mode.
    $this->assertVariationBaseValues($node, $entity_variations[0], 'en', $this->viewMode);
    // Check that only text1 field value is rendered and else fields are hidden.
    $this->assertStringNotContainsString($en_body_value, $entity_variations[0]['rendered_data']);
    $this->assertStringNotContainsString($en_text1_field_value, $entity_variations[0]['rendered_data']);

    // Custom1 view mode.
    $this->assertVariationBaseValues($node, $entity_variations[1], 'en', $custom1_view_mode_name);
    // Check that only text1 field value is rendered and else fields are hidden.
    $this->assertStringNotContainsString($en_body_value, $entity_variations[1]['rendered_data']);
    $this->assertStringNotContainsString($en_text1_field_value, $entity_variations[1]['rendered_data']);
  }

  /**
   * Use case 5.
   *
   * Test node with 1 view mode & multiple translations (en, fr).
   */
  public function testTranslationsAndCustomViewModes() {
    $translation_langcode = 'fr';
    ConfigurableLanguage::createFromLangcode($translation_langcode)->save();
    $this->drupalCreateContentType([
      'type' => $this->nodeBundle,
      'name' => $this->nodeBundle,
    ]);
    \Drupal::service('content_translation.manager')->setEnabled(
      $this->nodeEntityTypeId,
      $this->nodeBundle,
      TRUE
    );

    // Textfield custom 1.
    $custom1_field_name = 'field_custom1';
    $field_storage = $this->createTextFieldStorage(
      $this->nodeEntityTypeId,
      $custom1_field_name
    );
    $this->createTextField(
      $field_storage,
      $this->nodeEntityTypeId,
      $this->nodeBundle
    );
    $custom1_view_mode_name = 'custom1';
    // Create custom1 view mode + view display for new field.
    EntityViewMode::create([
      'id' => "{$this->nodeEntityTypeId}.{$custom1_view_mode_name}",
      'label' => 'Custom 1',
      'targetEntityType' => $this->nodeEntityTypeId,
    ])->save();
    EntityViewDisplay::create([
      'targetEntityType' => $this->nodeEntityTypeId,
      'bundle' => $this->nodeBundle,
      'mode' => $custom1_view_mode_name,
    ])->setStatus(TRUE)
      ->setComponent($custom1_field_name, [
        'label' => 'inline',
        'type' => 'string',
      ])
      ->save();

    $en_title = 'en news 1';
    $en_body_value = 'en news 1 body';
    $en_text1_field_value = 'en news 1 text 1';
    $fr_title = $translation_langcode . ' news 1';
    $fr_body_value = $translation_langcode . ' news 1 body';
    $fr_text1_field_value = $translation_langcode . ' news 1 text 1';

    $en_node = $this->drupalCreateNode([
      'type' => $this->nodeBundle,
      'title' => $en_title,
      'body' => ['value' => $en_body_value],
      $custom1_field_name => ['value' => $en_text1_field_value],
    ]);
    $en_node->save();
    $en_node->addTranslation($translation_langcode, [
      'title' => $fr_title,
      'body' => ['value' => $fr_body_value],
      $custom1_field_name => ['value' => $fr_text1_field_value],
    ])->save();
    $fr_node = $en_node->getTranslation($translation_langcode);

    $entity_types = [
      $this->nodeEntityTypeId => [
        $this->nodeBundle => [
          $this->viewMode => $this->viewModeDefaultValue,
          $custom1_view_mode_name => $this->viewModeDefaultValue,
        ],
      ],
    ];
    $this->setUpPerzEntityTypes($entity_types);

    $graphql_query_variables = [
      'entity_type_id' => $en_node->getEntityTypeId(),
      'entity_uuid' => $en_node->uuid(),
    ];

    $result = $this->sendQuery(
      $this->graphQLQuery,
      $this->graphqlServer,
      $graphql_query_variables
    );
    $this->assertEquals(200, $result->getStatusCode());
    $json_content = json_decode($result->getContent(), TRUE);

    $entity_variations = $json_content['data']['entity_variations'];

    // en, Default view mode.
    $this->assertVariationBaseValues($en_node, $entity_variations[0], 'en', $this->viewMode);
    // Check that only body field value is rendered and else fields are hidden.
    $this->assertStringNotContainsString($en_body_value, $entity_variations[0]['rendered_data']);
    $this->assertStringNotContainsString($en_text1_field_value, $entity_variations[0]['rendered_data']);

    // fr, Default view mode.
    $this->assertVariationBaseValues($fr_node, $entity_variations[1], $translation_langcode, $this->viewMode);
    // Check that only body field value is rendered and else fields are hidden.
    $this->assertStringNotContainsString($fr_body_value, $entity_variations[1]['rendered_data']);
    $this->assertStringNotContainsString($fr_text1_field_value, $entity_variations[1]['rendered_data']);

    // en, Custom1 view mode.
    $this->assertVariationBaseValues($en_node, $entity_variations[2], 'en', $custom1_view_mode_name);
    // Check that only text1 field value is rendered and else fields are hidden.
    $this->assertStringNotContainsString($en_body_value, $entity_variations[2]['rendered_data']);
    $this->assertStringNotContainsString($en_text1_field_value, $entity_variations[2]['rendered_data']);

    // fr, Custom1 view mode.
    $this->assertVariationBaseValues($fr_node, $entity_variations[3], $translation_langcode, $custom1_view_mode_name);
    // Check that only body field value is rendered and else fields are hidden.
    $this->assertStringNotContainsString($fr_body_value, $entity_variations[3]['rendered_data']);
    $this->assertStringNotContainsString($fr_text1_field_value, $entity_variations[3]['rendered_data']);

  }

  /**
   * Use case 6.
   *
   * Test taxonomy relations.
   */
  public function testTaxonomyRelations() {
    $this->drupalCreateContentType([
      'type' => $this->nodeBundle,
      'name' => $this->nodeBundle,
    ]);

    $tags_vocabulary = $this->createVocabulary();
    $materials_vocabulary = $this->createVocabulary();
    $tags_vocabulary_id = $tags_vocabulary->id();
    $materials_vocabulary_id = $materials_vocabulary->id();

    // Create tags taxonomy field.
    $tags_field_storage = $this->createTaxonomyTermFieldStorage(
      $this->nodeEntityTypeId,
      $this->tagsFieldName
    );
    $this->createTaxonomyTermField(
      $tags_field_storage,
      $this->nodeEntityTypeId,
      $this->nodeBundle,
      [
        $tags_vocabulary_id => $tags_vocabulary_id,
      ]
    );

    // Create materials taxonomy field.
    $materials_field_storage = $this->createTaxonomyTermFieldStorage(
      $this->nodeEntityTypeId,
      $this->materialsFieldName
    );
    $this->createTaxonomyTermField(
      $materials_field_storage,
      $this->nodeEntityTypeId,
      $this->nodeBundle,
      [
        $materials_vocabulary_id => $materials_vocabulary_id,
      ]
    );

    // Create categories taxonomy field.
    $categories_field_storage = $this->createTaxonomyTermFieldStorage(
      $this->nodeEntityTypeId,
      $this->categoriesFieldName
    );
    $this->createTaxonomyTermField(
      $categories_field_storage,
      $this->nodeEntityTypeId,
      $this->nodeBundle,
      [
        $tags_vocabulary_id => $tags_vocabulary_id,
        $materials_vocabulary_id => $materials_vocabulary_id,
      ]
    );

    // Create tags terms.
    $tag1 = $this->createTerm($tags_vocabulary);
    $tag2 = $this->createTerm($tags_vocabulary);
    $tag3 = $this->createTerm($tags_vocabulary);

    // Create materials terms.
    $material1 = $this->createTerm($materials_vocabulary);
    $material2 = $this->createTerm($materials_vocabulary);
    $material3 = $this->createTerm($materials_vocabulary);

    $node1 = $this->drupalCreateNode([
      'type' => $this->nodeBundle,
      'title' => 'News 1',
      $this->tagsFieldName => [
        ['target_id' => $tag1->id()],
        ['target_id' => $tag2->id()],
      ],
    ]);
    $node1->save();

    $node2 = $this->drupalCreateNode([
      'type' => $this->nodeBundle,
      'title' => 'News 2',
      $this->tagsFieldName => [
        ['target_id' => $tag1->id()],
        ['target_id' => $tag2->id()],
      ],
      $this->materialsFieldName => [
        ['target_id' => $material1->id()],
      ],
    ]);
    $node2->save();

    $node3 = $this->drupalCreateNode([
      'type' => $this->nodeBundle,
      'title' => 'News 3',
      $this->tagsFieldName => [
        ['target_id' => $tag1->id()],
        ['target_id' => $tag2->id()],
      ],
      $this->materialsFieldName => [
        ['target_id' => $material1->id()],
      ],
      $this->categoriesFieldName => [
        ['target_id' => $tag3->id()],
        ['target_id' => $material2->id()],
      ],
    ]);
    $node3->save();

    $node4 = $this->drupalCreateNode([
      'type' => $this->nodeBundle,
      'title' => 'News 4',
      $this->tagsFieldName => [
        ['target_id' => $tag1->id()],
        ['target_id' => $tag2->id()],
      ],
      $this->materialsFieldName => [
        ['target_id' => $material1->id()],
        ['target_id' => $material2->id()],
        ['target_id' => $material3->id()],
      ],
      $this->categoriesFieldName => [
        ['target_id' => $tag1->id()],
        ['target_id' => $material2->id()],
        ['target_id' => $tag2->id()],
        ['target_id' => $material3->id()],
      ],
    ]);
    $node4->save();

    // Setup all taxonomy terms for tracking.
    $this->setUpPerzEntityTypes([
      'taxonomy_term' => [
        $tags_vocabulary_id => [
          $this->viewMode => $this->viewModeDefaultValue,
        ],
        $materials_vocabulary_id => [
          $this->viewMode => $this->viewModeDefaultValue,
        ],
      ],
      $this->nodeEntityTypeId => [
        $this->nodeBundle => [
          $this->viewMode => $this->viewModeDefaultValue,
        ],
      ],
    ]);

    // Check relations for node1.
    $this->assertRelationsByEntity($node1, [
      [
        'field' => 'field_tags',
        'terms' => [
          $tag1->uuid(),
          $tag2->uuid(),
        ],
      ],
    ]);
    // Check relations for node2.
    $this->assertRelationsByEntity($node2,
      [
        [
          'field' => 'field_materials',
          'terms' => [
            $material1->uuid(),
          ],
        ],
        [
          'field' => 'field_tags',
          'terms' => [
            $tag1->uuid(),
            $tag2->uuid(),
          ],
        ],
      ]
    );
    // Check relations for node3.
    $this->assertRelationsByEntity($node3,
      [
        [
          'field' => 'field_categories',
          'terms' => [
            $tag3->uuid(),
            $material2->uuid(),
          ],
        ],
        [
          'field' => 'field_materials',
          'terms' => [
            $material1->uuid(),
          ],
        ],
        [
          'field' => 'field_tags',
          'terms' => [
            $tag1->uuid(),
            $tag2->uuid(),
          ],
        ],
      ]
    );
    // Check relations for node4.
    $this->assertRelationsByEntity($node4,
      [
        [
          'field' => 'field_categories',
          'terms' => [
            $material3->uuid(),
            $tag2->uuid(),
            $material2->uuid(),
            $tag1->uuid(),
          ],
        ],
        [
          'field' => 'field_materials',
          'terms' => [
            $material1->uuid(),
            $material2->uuid(),
            $material3->uuid(),
          ],
        ],
        [
          'field' => 'field_tags',
          'terms' => [
            $tag1->uuid(),
            $tag2->uuid(),
          ],
        ],
      ]
    );
    // Test relations when 1 taxonomy (materials) is disabled.
    $this->setUpPerzEntityTypes([
      $this->nodeEntityTypeId => [
        $this->nodeBundle => [
          $this->viewMode => $this->viewModeDefaultValue,
        ],
      ],
      'taxonomy_term' => [
        $tags_vocabulary_id => [
          $this->viewMode => $this->viewModeDefaultValue,
        ],
      ],
    ]);
    // Check relations for node1.
    $this->assertRelationsByEntity($node1, [
      [
        'field' => 'field_tags',
        'terms' => [
          $tag1->uuid(),
          $tag2->uuid(),
        ],
      ],
    ]);
    // Check relations for node2.
    $this->assertRelationsByEntity($node2, [
      [
        'field' => 'field_tags',
        'terms' => [
          $tag1->uuid(),
          $tag2->uuid(),
        ],
      ],
    ]);
    // Check relations for node3.
    $this->assertRelationsByEntity($node3, [
      [
        'field' => 'field_categories',
        'terms' => [
          $tag3->uuid(),
        ],
      ],
      [
        'field' => 'field_tags',
        'terms' => [
          $tag1->uuid(),
          $tag2->uuid(),
        ],
      ],
    ]);
    // Check relations for node4.
    $this->assertRelationsByEntity($node4, [
      [
        'field' => 'field_categories',
        'terms' => [
          $tag2->uuid(),
          $tag1->uuid(),
        ],
      ],
      [
        'field' => 'field_tags',
        'terms' => [
          $tag1->uuid(),
          $tag2->uuid(),
        ],
      ],
    ]);
    // Test relations when 1 taxonomy (tags) is disabled.
    $this->setUpPerzEntityTypes([
      $this->nodeEntityTypeId => [
        $this->nodeBundle => [
          $this->viewMode => $this->viewModeDefaultValue,
        ],
      ],
      'taxonomy_term' => [
        $materials_vocabulary_id => [
          $this->viewMode => $this->viewModeDefaultValue,
        ],
      ],
    ]);
    // Check relations for node1.
    $this->assertRelationsByEntity($node1, []);
    // Check relations for node2.
    $this->assertRelationsByEntity($node2, [
      [
        'field' => 'field_materials',
        'terms' => [
          $material1->uuid(),
        ],
      ],
    ]);
    // Check relations for node3.
    $this->assertRelationsByEntity($node3, [
      [
        'field' => 'field_categories',
        'terms' => [
          $material2->uuid(),
        ],
      ],
      [
        'field' => 'field_materials',
        'terms' => [
          $material1->uuid(),
        ],
      ],
    ]);
    // Check relations for node4.
    $this->assertRelationsByEntity($node4, [
      [
        'field' => 'field_categories',
        'terms' => [
          $material3->uuid(),
          $material2->uuid(),
        ],
      ],
      [
        'field' => 'field_materials',
        'terms' => [
          $material1->uuid(),
          $material2->uuid(),
          $material3->uuid(),
        ],
      ],
    ]);
    // Test relations when all taxonomies are disabled.
    $this->setUpPerzEntityTypes([
      $this->nodeEntityTypeId => [
        $this->nodeBundle => [
          $this->viewMode => $this->viewModeDefaultValue,
        ],
      ],
    ]);
    // Check relations for node1.
    $this->assertRelationsByEntity($node1, []);
    // Check relations for node2.
    $this->assertRelationsByEntity($node2, []);
    // Check relations for node3.
    $this->assertRelationsByEntity($node3, []);
    // Check relations for node4.
    $this->assertRelationsByEntity($node4, []);
  }

  /**
   * Send graphql request and assert taxonomy relations per entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The target entity that contains taxonomy fields.
   * @param array $expected_relations
   *   Expected relations.
   *
   * @throws \Exception
   */
  protected function assertRelationsByEntity(
    EntityInterface $entity,
    array $expected_relations
  ) {
    // Check relations for node3.
    $graphql_query_variables = [
      'entity_type_id' => $entity->getEntityTypeId(),
      'entity_uuid' => $entity->uuid(),
    ];
    $result = $this->sendQuery(
      $this->graphQLQuery,
      $this->graphqlServer,
      $graphql_query_variables
    );
    $this->assertEquals(200, $result->getStatusCode());
    $json_content = json_decode($result->getContent(), TRUE);
    $variations = $json_content['data']['entity_variations'];
    $actual_relations = $variations[0]['relations'];
    if ($actual_relations && $expected_relations) {
      $this->assertEqualsCanonicalizing(
        $actual_relations,
        $expected_relations
      );
    }
    else {
      $this->assertEmpty((array) $actual_relations);
      $this->assertEmpty($expected_relations);
    }
  }

}
